﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Xml;

namespace Mapa
{
    public partial class Form1 : Form
    {
        Image[,] images;
        bool canDraw = false;
        decimal oldZoomValue;
        int lastDelta;
        bool mouseDown;
        Point offset = new Point(0, 0), lastOffset = new Point(0, 0),
            clickedMousePos = new Point(0, 0);

        int Zoom { get; set; }
        int X { get; set; }
        int Y { get; set; }

        List<Obec> obce = new List<Obec>();
        List<Obec> searchResults = new List<Obec>();

        public Form1()
        {
            InitializeComponent();
        }

        private void bZobraz_Click(object sender, EventArgs e)
        {
            canDraw = false;

            try
            {
                images = new Image[7, 7];

                //načíst mřížku obrázků 5*5
                for (int i = -3; i < 4; i++)
                {
                    for (int j = -3; j < 4; j++)
                    {
                        if (i + X < 0 || j + Y < 0 || i + X > 3 * Math.Pow(2, Zoom) - 1 || j + Y > 2 * Math.Pow(2, Zoom) - 1)
                        {
                            images[i + 3, j + 3] = null;
                            continue;
                        }

                        //webrequest
                        WebRequest request = WebRequest.Create("http://" + tServer.Text + ":" + tPort.Text
                            + "/" + Zoom.ToString() + "/" + (X + i).ToString() + "/" + (Y + j).ToString() + ".jpg");
                        WebResponse response = request.GetResponse();
                        Stream stream = response.GetResponseStream();
                        images[i + 3, j + 3] = Image.FromStream(stream);
                        stream.Close();
                        response.Close();
                    }
                }

                canDraw = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "Chyba");
            }

            this.Refresh();
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            if (!canDraw)
                return;

            Graphics g = e.Graphics;

            for (int i = 0; i < 7; i++)
            {
                for (int j = 0; j < 7; j++)
                {
                    //vykreslit mřížku obrázků
                    if (images[i, j] != null)
                        g.DrawImage(images[i, j], new Rectangle((i - 3) * 256 + this.Width / 2 - 128 + offset.X,
                            (j - 3) * 256 + this.Height / 2 - 128 + offset.Y, 256, 256));
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            oldZoomValue = Zoom;
            X = 1;
            Y = 1;

            bObce_Click(null, null);
            bZobraz_Click(null, null);

            this.MouseWheel += new MouseEventHandler(Form1_MouseWheel);
        }

        //zoom kolečkem myši
        void Form1_MouseWheel(object sender, MouseEventArgs e)
        {
            if (lastDelta < e.Delta)
                bZoom_Click(null, null);
            else if (lastDelta > e.Delta)
                bUnZoom_Click(null, null);

            lastDelta = e.Delta;
        }

        private void Form1_MouseDown(object sender, MouseEventArgs e)
        {
            //zapnout posouvání
            mouseDown = true;
            clickedMousePos = new Point(e.X, e.Y);

            lastOffset.X = offset.X;
            lastOffset.Y = offset.Y;
        }

        private void Form1_MouseMove(object sender, MouseEventArgs e)
        {
            //pohyb mapou
            if (mouseDown)
            {
                offset.X = lastOffset.X + e.X - clickedMousePos.X;
                offset.Y = lastOffset.Y + e.Y - clickedMousePos.Y;
                this.Refresh();
            }
        }

        private void Form1_MouseUp(object sender, MouseEventArgs e)
        {
            //ukončit posouvání
            mouseDown = false;
            X -= offset.X / 256;
            Y -= offset.Y / 256;
            offset.X = offset.X % 256;
            offset.Y = offset.Y % 256;

            bZobraz_Click(null, null);
        }

        private void ValidateXYZ()
        {
            //validace zoomu
            if (Zoom < 0)
                Zoom = 0;
            else if (Zoom > 9)
                Zoom = 9;

            //validace souřadnic
            if (X < 0)
                X = 0;
            if (X > 3 * Math.Pow(2, Zoom))
                X = (int)(3 * Math.Pow(2, Zoom));

            if (Y < 0)
                Y = 0;
            if (Y > 3 * Math.Pow(2, Zoom))
                Y = (int)(3 * Math.Pow(2, Zoom));
        }

        //zoom
        private void bZoom_Click(object sender, EventArgs e)
        {
            if (Zoom < 9)
            {
                bZoom.Enabled = false;
                bUnZoom.Enabled = false;

                Zoom++;
                lZoom.Text = Zoom.ToString();
                
                X *= 2;
                Y *= 2;
                
                offset.X *= 2;
                offset.Y *= 2;
                
                bZobraz_Click(null, null);

                bZoom.Enabled = true;
                bUnZoom.Enabled = true;
            }
        }

        //unzoom
        private void bUnZoom_Click(object sender, EventArgs e)
        {
            if (Zoom > 0)
            {
                bZoom.Enabled = false;
                bUnZoom.Enabled = false;

                Zoom--;
                lZoom.Text = Zoom.ToString();

                X /= 2;
                Y /= 2;

                offset.X /= 2;
                offset.Y /= 2;

                bZobraz_Click(null, null);

                bZoom.Enabled = true;
                bUnZoom.Enabled = true;
            }
        }

        private void bObce_Click(object sender, EventArgs e)
        {
            bObce.Enabled = false;

            //webrequest
            WebRequest request = WebRequest.Create("http://" + tServer.Text + ":" + tPort.Text + "/obce.xml");
            WebResponse response = request.GetResponse();
            Stream stream = response.GetResponseStream();

            XmlTextReader reader = new XmlTextReader(stream);

            //načíst obce
            while (reader.Read())
            {
                if (reader.NodeType == XmlNodeType.Element && reader.Name=="obec")
                {
                    //zapsat obec
                    obce.Add(new Obec(reader.GetAttribute(0).ToString(), Convert.ToInt32(reader.GetAttribute(1)),
                        Convert.ToInt32(reader.GetAttribute(2))));
                }
            }

            stream.Close();
            response.Close();

            bObce.Enabled = true;
        }

        //hledání obcí
        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            lHledani.Items.Clear();
            searchResults.Clear();

            foreach (Obec obec in obce)
            {
                if (obec.Name.ToLower().Contains(tHledani.Text.ToLower()))
                {
                    lHledani.Items.Add(obec.Name);
                    searchResults.Add(obec);
                }
            }
        }

        //zobrazení obce
        private void lHledani_SelectedIndexChanged(object sender, EventArgs e)
        {
            Obec result = searchResults[lHledani.SelectedIndex];
            offset.X = (int)(result.X / (Math.Pow(2, 9 - Zoom) / -1.0));
            offset.Y = (int)(result.Y / (Math.Pow(2, 9 - Zoom) / -1.0));
            X = 0; Y = 0;

            Form1_MouseUp(null, null);
        }
    }

    //třída obec pro uložení dat
    public class Obec
    {
        public string Name { get; set; }
        public int X { get; set; }
        public int Y { get; set; }

        public Obec(string name, int x, int y)
        {
            this.Name = name;
            this.X = x;
            this.Y = y;
        }
    }
}